import logging

from django.conf import settings as django_settings

from zapier.triggers.models import TriggerSubscription

logger = logging.getLogger(__name__)


def subscribe(
    user: django_settings.AUTH_USER_MODEL, trigger: str, target_url: str
) -> TriggerSubscription:
    """Create a new REST Hook subscription."""
    # when a zap is disabled the subscription is unsubscribed - in this
    # instance we have an inactive subscription, so we update the target
    # url (it will be different) and reset the timestamps.
    logger.debug("Creating new subscription for trigger '%s'", trigger)
    if subscription := TriggerSubscription.objects.filter(
        trigger=trigger,
        user=user,
    ).last():
        subscription.resubscribe(target_url)
    else:
        subscription = TriggerSubscription.objects.create(
            trigger=trigger,
            user=user,
            target_url=target_url,
        )
    return subscription


def unsubscribe(subscription: TriggerSubscription) -> None:
    """Delete a RestHookSubscription."""
    logger.debug("Deleting subscription '%s'", subscription.uuid)
    subscription.unsubscribe()



import json
import logging
import uuid
from typing import Callable

from django.conf import settings
from django.core.exceptions import MiddlewareNotUsed
from django.http import HttpRequest, HttpResponse

logger = logging.getLogger(__name__)


def dump_json(data: bytes) -> str:
    if not data:
        return "(empty)"
    try:
        return json.dumps(json.loads(data), indent=2, sort_keys=True)
    except json.JSONDecodeError:
        logger.exception("Error decoding data")
        return "(error)"


class JsonRequestDumpMiddleware:
    def __init__(self, get_response: Callable) -> None:
        if not settings.DEBUG:
            raise MiddlewareNotUsed(
                "JsonRequestDumpMiddleware can only run in DEBUG mode."
            )
        self.get_response = get_response

    def __call__(self, request: HttpRequest) -> HttpResponse:
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        request_id = str(uuid.uuid4()).split("-", 1)[0].upper()
        logger.debug(
            "%s - request headers:\n%s",
            request_id,
            json.dumps(dict(request.headers), indent=2, sort_keys=True),
        )
        if request.headers.get("content-type", "") == "application/json":
            logger.debug("%s - request body:\n%s", request_id, dump_json(request.body))
        response = self.get_response(request)
        if response.headers.get("content-type", "") == "application/json":
            logger.debug(
                "%s - response content:\n%s", request_id, dump_json(response.content)
            )
        return response



from __future__ import annotations

import json
import logging

from django.contrib import admin
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.html import format_html, mark_safe

from .models import TriggerEvent, TriggerSubscription

logger = logging.getLogger(__name__)


def format_json_for_admin(data: dict) -> str:
    """
    Pretty-print JSON data in admin site.

    Take entity context JSON, indent it, order the keys and then present
    it as a <code> block. That's about as good as we can get until
    someone builds a custom syntax function.

    """
    pretty = json.dumps(
        data, cls=DjangoJSONEncoder, sort_keys=True, indent=4, separators=(",", ": ")
    )
    # https://docs.djangoproject.com/en/1.11/ref/utils/#django.utils.html.format_html
    # this is a fudge to get around the fact that we cannot put a <pre> inside a <p>,
    # but we want the <p> formatting (.align CSS). We can either use a <pre> and an
    # inline style to mimic the CSS, or forego the <pre> and put the spaces
    # and linebreaks in as HTML.
    pretty = pretty.replace(" ", "&nbsp;").replace("\n", "<br/>")
    return format_html("<code>{}</code>", mark_safe(pretty))


@admin.register(TriggerSubscription)
class TriggerSubscriptionAdmin(admin.ModelAdmin):
    list_display = ("user", "trigger", "subscribed_at", "_is_active")
    readonly_fields = (
        "user",
        "trigger",
        "target_url",
        "subscribed_at",
        "unsubscribed_at",
        "uuid",
    )

    @admin.display(boolean=True)
    def _is_active(self, obj: TriggerSubscription) -> bool:
        return obj.is_active


@admin.register(TriggerEvent)
class TriggerEventAdmin(admin.ModelAdmin):

    list_display = (
        "user",
        "trigger",
        "http_method",
        "started_at",
        "duration",
        "status_code",
        # "status",
    )
    readonly_fields = (
        "user",
        "trigger",
        "http_method",
        "started_at",
        "finished_at",
        "duration",
        "subscription",
        "status_code",
        "_event_data",
    )
    exclude = ("event_data",)
    raw_id_fields = ("user", "subscription")
    search_fields = (
        "user__first_name",
        "user__last_name",
    )

    @admin.display(description="Event data")
    def _event_data(self, obj: TriggerEvent) -> str:
        return format_json_for_admin(obj.event_data)



# Generated by Django 4.1 on 2022-09-12 14:56

import uuid

import django.core.serializers.json
import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="TriggerSubscription",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("uuid", models.UUIDField(default=uuid.uuid4, help_text="Public ID")),
                (
                    "trigger",
                    models.CharField(
                        db_index=True,
                        help_text="The name of the trigger event to subscribe to.",
                        max_length=50,
                    ),
                ),
                (
                    "target_url",
                    models.URLField(
                        help_text="The webhook URL to which any payload will be POSTed."
                    ),
                ),
                (
                    "subscribed_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Timestamp marking when the initial subscribe event occurred.",
                    ),
                ),
                (
                    "unsubscribed_at",
                    models.DateTimeField(
                        blank=True,
                        default=None,
                        help_text="Timestamp marking when the unsubscribe event occurred.",
                        null=True,
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="zapier_sbuscriptions",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="TriggerEvent",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "trigger",
                    models.CharField(
                        help_text="The name of the Zapier trigger.", max_length=100
                    ),
                ),
                (
                    "http_method",
                    models.CharField(
                        help_text="How the data was sent to Zapier - via GET, or POST.",
                        max_length=4,
                    ),
                ),
                (
                    "event_data",
                    models.JSONField(
                        blank=True,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                        help_text="JSON data sent to Zapier.",
                        null=True,
                    ),
                ),
                (
                    "started_at",
                    models.DateTimeField(
                        blank=True, default=django.utils.timezone.now, null=True
                    ),
                ),
                ("finished_at", models.DateTimeField(blank=True, null=True)),
                (
                    "status_code",
                    models.IntegerField(
                        help_text="HTTP status code received from Zapier"
                    ),
                ),
                (
                    "subscription",
                    models.ForeignKey(
                        blank=True,
                        help_text="The subscription to which the event was posted (null for polling events).",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="trigger_events",
                        to="zapier_triggers.triggersubscription",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="zapier_trigger_events",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



import importlib
from typing import Callable, Type

from django.conf import settings as django_settings
from django.core.exceptions import ImproperlyConfigured
from rest_framework.authentication import BaseAuthentication

from .types import TriggerViewFunc


def import_from_path(path: str) -> Type | Callable:
    """Import function from string path."""
    module, func_or_klass = path.rsplit(".", 1)
    m = importlib.import_module(module)
    return getattr(m, func_or_klass)


_settings = getattr(django_settings, "ZAPIER_TRIGGERS", {})

# set to True to reject requests that don't come from Zapier
STRICT_MODE = bool(_settings.get("STRICT_MODE", not django_settings.DEBUG))

# the JSON key used to extract the REST Hook url from inbound post ("hookUrl")
HOOK_URL_KEY = _settings.get("HOOK_URL_KEY", "hookUrl")


def get_authenticator() -> BaseAuthentication:
    """Return authentication class imported from AUTHENTICATOR setting."""
    if not (path := _settings.get("AUTHENTICATOR")):
        raise ImproperlyConfigured("Missing AUTHENTICATOR.")
    return import_from_path(path)


def get_trigger_data_func(trigger: str) -> TriggerViewFunc:
    """Return view data function configured for the trigger."""
    if not (func_path := _settings.get("GET_DATA_FUNCS", {}).get(trigger)):
        raise ImproperlyConfigured("Missing trigger view function.")
    return import_from_path(func_path)


def trigger_exists(trigger: str) -> bool:
    """Check that the trigger is configured."""
    return trigger in _settings.get("GET_DATA_FUNCS", {})



import logging

import requests
from django.utils.timezone import now as tz_now

from .models import TriggerEvent, TriggerSubscription

logger = logging.getLogger(__name__)


def push(subscription: TriggerSubscription, event_data: dict) -> TriggerEvent:
    """Push data to Zapier."""
    logger.debug("Pushing webhook data:\n%s", event_data)
    started_at = tz_now()
    response = requests.post(subscription.target_url, json=event_data)
    finished_at = tz_now()
    return TriggerEvent.objects.create(
        user=subscription.user,
        trigger=subscription.trigger,
        subscription=subscription,
        started_at=started_at,
        finished_at=finished_at,
        http_method="POST",
        event_data=event_data,
        status_code=response.status_code,
    )



from django.apps import AppConfig


class ZapierTriggerConfig(AppConfig):

    name = "zapier.triggers"
    label = "zapier_triggers"
    verbose_name = "Zapier Triggers"
    default_auto_field = "django.db.models.BigAutoField"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zapier',
 'zapier.triggers',
 'zapier.triggers.migrations',
 'zapier.triggers.models']

package_data = \
{'': ['*']}

install_requires = \
['django>=3.2,<5.0',
 'djangorestframework',
 'python-dateutil',
 'pytz',
 'requests']

setup_kwargs = {
    'name': 'django-zapier-trigger',
    'version': '1.0a3',
    'description': 'Django (DRF) backed app for managing Zapier triggers.',
    'long_description': '# Django Zapier Triggers\n\n**DO NOT USE IN PRODUCTION - ALPHA RELEASE**\n\nDjango app for managing Zapier triggers.\n\n### Version support\n\nThis app supports Django 3.2+ (`HttpResponse.headers`), and Python 3.10+.\n\nThis app provides the minimal scaffolding required to support a Zapier\ntrigger in your application. Specifically it supports token-based\nauthentication and endpoints for RestHook and Polling triggers.\n\nAs well as three Django apps (`zapier.contrib.authtoken`,\n`zapier.triggers.hooks` and `zapier.triggers.polling`) this project also\nincludes a working Zapier app (publishable using the Zapier CLI), and a\nDemo django project that you can use to test the Zapier integration.\nWith the Demo app running locally (and available to the internet via\ne.g. ngrok) you can test pushing data to a Zapier "zap".\n\n### Prequisites\n\nIf you want to run the end-to-end demo you will need:\n\n1. A Zapier account\n2. The Zapier CLI\n3. ngrok, or some equivalent tunnelling software\n\n## How does it work?\n\n\n\n## Usage\n',
    'author': 'YunoJuno',
    'author_email': 'code@yunojuno.com',
    'maintainer': 'YunoJuno',
    'maintainer_email': 'code@yunojuno.com',
    'url': 'https://github.com/yunojuno/django-zapier-trigger',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
